function [y,w] = eigBSE( X, tmdelay, selectionIC, trace)
% eigBSE-Temporally correlated signal extraction based on eigenvalue decomposition
%        It extracts a source signal, exploiting its special
%        autocorrelation information. So, before running this algorithm,
%        one should know the desired source's autocorrelation information,
%        i.e. the optimal time delay t^*:
%              E{ S_i * S_i(t-t^*) }  not.equ. 0
%              E{ S_j * S_j(t-t^*) }  equ. 0 .
%        or, strictly but practically:
%              E{ S_i * S_i(t-t^*) } > E{ S_j * S_j(t-t^*) }.
%        It returns the desired source y and corresponding weight vector w(y=w'*X). 
%        If there are several source signals having the same period length T, then
%        the algorithm still extracts these signals one by one, providing that they
%        have different autocorrelation values at lag T. For details see
%        Ref.[2].
%      *** Note the input observed mixed signals X should be prewhitened ***
%
% Command:
%    [y,w] = eigBSE( X, tmdelay, selectionIC, trace)
%    [y,w] = eigBSE( X, tmdelay, selectionIC)
%
% Parameters:
%          y --- Extracted source signal
%          w --- Corresponding weight column vector, that is, y=w'*X.
%          X --- Prewhitened observed mixed signals. Each row is a mixed signal.            
%    tmdelay --- A vector. Its each element represents an optimal time delay, i.e., 
%                the autocorrelation value at the time delay tmdelay is far from zero, 
%                while the other sources' autocorrelation value at the time delay is nearly
%                zero.
% selectionIC -- A vector, whose element indicates extracting the corresponding source. 
%                For example, say selectionIC(3) = k. Then the algorithm
%                extracts the source corresponding to the k-th largest
%                eigenvalues of E{S(t)*S(t-t^*)}.
%      trace --- Print out the progress. 1 - print; 0 - not print.
%      
%
% Reference:
%    [1] Zhi-Lin Zhang, Zhang Yi, Robust  Extraction of Specific Signals with 
%        Temporal Structure, Neurocomputing 69 (7-9) 888-893 
%    [2] Zhi-Lin Zhang, Morphologically Constrained ICA for Extracting Weak 
%        Temporally Correlated Signals, Neurocomputing 71(7-9) (2008)
%        1669-1679 
%
% Author: Zhilin Zhang (zhang@sccn.ucsd.edu)
% Date: July 2005


[ICnum, IClen]=size(X);

if nargin == 3,   trace = 1;  end
if trace == 1,   fprintf('\n\nStarting eigBSE ...\n');  end


%------------------------------------------------------------------------------------
% compute the multiple delayed covariance matrix of observed data
Rx = zeros(size(X,1));
for k = tmdelay
    tempo = X(:,k+1:IClen) * X(:,1: IClen-k)' / (IClen-k);
    Rx = Rx + tempo;
end
clear tempo;
Rx = (Rx + Rx')/2;
[V,D] = eig(Rx);   
w = V(:,ICnum-selectionIC+1);
y = w'* X;

d = diag(D);

if trace == 1
    for l = 1:length(selectionIC)
        fprintf('Extracting the source corresponding to the %d-th largest eigenvalue.\n    The eigenvalue is %g\n', ...
            selectionIC(l),d(ICnum-selectionIC(l)+1));
    end
end

% eigs = flipud(diag(D));
% numSource = find( eigs> -eigs(1) * 0.1 );
% if trace==1  
%     fprintf('The first largest %d eigenvalues are :\n',numSource(end));
%     disp(eigs(1:numSource(end)));
%     fprintf('Note: There may be %d source signals with the same autocorrelation value\n',numSource(end));   
%     fprintf('      And you are extracting the No.%d source signal.\n',selectionIC);
% end

if trace==1  fprintf('End of eigBSE!\n\n');   end


